<?php
/**
 * Created by PhpStorm.
 * User: crose
 * Date: 3/13/17
 * Time: 9:29 PM
 */

namespace qfq;

//use TYPO3\CMS\Core\FormProtection\Exception;

require_once(__DIR__ . '/../store/Sip.php');
require_once(__DIR__ . '/../store/Session.php');
require_once(__DIR__ . '/../Constants.php');
require_once(__DIR__ . '/../helper/Ldap.php');
require_once(__DIR__ . '/../database/Database.php');


class TypeAhead {

    /**
     * @var Database instantiated class
     */
    protected $db = null;

    /**
     * @var array
     */
    protected $vars = array();

    /**
     *
     */
    public function __construct($phpUnit = false) {

        $this->vars[TYPEAHEAD_API_QUERY] = isset($_GET[TYPEAHEAD_API_QUERY]) ? $_GET[TYPEAHEAD_API_QUERY] : '';
        $this->vars[TYPEAHEAD_API_PREFETCH] = isset($_GET[TYPEAHEAD_API_PREFETCH]) ? $_GET[TYPEAHEAD_API_PREFETCH] : '';
        $this->vars[TYPEAHEAD_API_SIP] = isset($_GET[TYPEAHEAD_API_SIP]) ? $_GET[TYPEAHEAD_API_SIP] : '';

        if ($this->vars[TYPEAHEAD_API_SIP] == '') {
            throw new CodeException('Missing GET parameter "' . TYPEAHEAD_API_SIP);
        }

        if ($this->vars[TYPEAHEAD_API_QUERY] == '' && $this->vars[TYPEAHEAD_API_PREFETCH] == '') {
            throw new CodeException('Missing GET parameter "' . TYPEAHEAD_API_QUERY . '" or "' . TYPEAHEAD_API_PREFETCH . '"');
        }

        $session = Session::getInstance($phpUnit);

        $this->db = new Database();

    }

    /**
     * @return array|int
     * @throws CodeException
     * @throws DbException
     * @throws UserFormException
     */
    public function process() {

        $arr = array();
        $values = array();

        $sipClass = new Sip();

        $sipVars = $sipClass->getVarsFromSip($this->vars[TYPEAHEAD_API_SIP]);

        if (isset($sipVars[FE_TYPEAHEAD_SQL])) {
            $arr = $this->typeAheadSql($sipVars, $this->vars[TYPEAHEAD_API_QUERY]);
        } elseif (isset($sipVars[FE_LDAP_SERVER])) {
            $ldap = new Ldap();

            if ($this->vars[TYPEAHEAD_API_PREFETCH] == '') {
                $mode = MODE_LDAP_MULTI;
                $key = $this->vars[TYPEAHEAD_API_QUERY];
            } else {
                $mode = MODE_LDAP_PREFETCH;
                $key = $this->vars[TYPEAHEAD_API_PREFETCH];
            }

            $arr = $ldap->process($sipVars, $key, $mode);
        }

        return $arr;
    }

    /**
     * Do a wildcard serach on the prepared statement $config[FE_TYPEAHEAD_SQL].
     * All '?' will be replaced by '%$value%'.
     * If there is no 'LIMIT x' defined, append it.
     * Returns an dict array [ API_TYPEAHEAD_KEY => key, API_TYPEAHEAD_VALUE => value ]
     *
     * @param array $config
     * @param string $value
     *
     * @return array
     * @throws CodeException
     * @throws DbException
     * @throws UserFormException
     */
    private function typeAheadSql(array $config, $value) {
        $values = array();

        $sql = $config[FE_TYPEAHEAD_SQL];
        $value = '%' . $value . '%';
        $cnt = substr_count($sql, '?');

        if ($cnt == 0) {
            throw new UserFormException("Missing at least one '?' in " . FE_TYPEAHEAD_SQL);
        }

        for ($ii = 0; $ii < $cnt; $ii++) {
            $values[] = $value;
        }

        if (!$this->db->hasLimit($sql)) {
            $sql .= ' LIMIT ' . $config[FE_TYPEAHEAD_LIMIT];
        }

        $arr = $this->db->sql($sql, ROW_REGULAR, $values);
        if ($arr == false || count($arr) == 0) {
            return array();
        }

        return $this->db->makeArrayDict($arr, TYPEAHEAD_SQL_KEY_NAME, API_TYPEAHEAD_VALUE, API_TYPEAHEAD_KEY, API_TYPEAHEAD_VALUE);
    }


}