/**
 * @author Rafael Ostertag <rafael.ostertag@math.uzh.ch>
 */

/* global console */
/* global CodeMirror */
/* global $ */

/**
 * Qfq Namespace
 *
 * @namespace QfqNS
 */
var QfqNS = QfqNS || {};
/**
 * Qfq Helper Namespace
 *
 * @namespace QfqNS.Helper
 */
QfqNS.Helper = QfqNS.Helper || {};

(function (n) {
    'use strict';

    /**
     * Initializes codemirror. Only `<textarea/>` elements having the class `qfq-codemirror` are initialized.
     *
     * The codemirror configuration has to be provided in the `data-config` attribute as JSON. E.g.
     *
     *      <textarea class="qfq-codemirror" data-config='{ "mode": "text/x-sql", "lineNumbers": true }'></textarea>
     *
     * @function
     */
    var codemirror = function () {
        if (typeof CodeMirror === 'undefined') {
            //QfqNS.log.error("CodeMirror not loaded, cannot initialize Qfq CodeMirror.");
            return;
        }

        $("textarea.qfq-codemirror").each(
            function () {
                var config = {};
                var $this = $(this);
                var configData = $this.data('config');
                if (configData) {
                    if (configData instanceof Object) {
                        // jQuery takes care of decoding data-config to JavaScript object.
                        config = configData;
                    } else {
                        QfqNS.Log.warning("'data-config' is invalid: " + configData);
                    }
                }
                var cm = CodeMirror.fromTextArea(this, configData);
                cm.on('change', (function ($form, $textArea) {
                    return function (instance, changeObj) {
                        $textArea.val(instance.getValue());
                        $form.change();
                    };
                })($(this).closest('form'), $this));
            }
        );
    };

    n.codemirror = codemirror;


})(QfqNS.Helper);